package com.github.gkutiel.filter;

import static com.github.gkutiel.filter.Auth.sign;
import static com.github.gkutiel.filter.Auth.verify;
import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.io.IOException;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.Gson;

public abstract class Filter {
	@Inherited
	@Target(FIELD)
	@Retention(RUNTIME)
	public @interface Cookie {}

	@Inherited
	@Target(FIELD)
	@Retention(RUNTIME)
	public @interface Param {}

	static class RedirectException extends RuntimeException {}

	static class SignedValue {
		private final String val;
		private final String sign;

		SignedValue(final String val, final String sign) {
			this.val = val;
			this.sign = sign;
		}

		boolean isValid() {
			return verify(val, sign);
		}

		String val() {
			return val;
		}
	}

	protected static final Gson GSON = new Gson();

	static String signedValue(final Object val) {
		final String json = GSON.toJson(val);
		return GSON.toJson(new SignedValue(json, sign(json)));
	}

	HttpServletRequest req;
	HttpServletResponse res;

	public abstract void apply();

	public <T> T redirect(final String path) {
		try {
			res.sendRedirect(path);
			throw new RedirectException();
		} catch (final IOException e) {
			throw new RuntimeException(e);
		}
	}
	protected void del(final String key) {
		set(key, null, 0);
	}

	protected <T> void set(final String key, final T val) {
		set(key, val, Integer.MAX_VALUE);
	}

	protected <T> void set(final String key, final T val, final int maxAgeInSec) {
		final javax.servlet.http.Cookie cookie = new javax.servlet.http.Cookie(key, signedValue(val));
		cookie.setMaxAge(maxAgeInSec);
		res.addCookie(cookie);
	}
}
