package com.github.gkutiel.filter;

import static com.github.gkutiel.filter.Util.getCookie;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.github.gkutiel.filter.Filter.Cookie;
import com.github.gkutiel.filter.Filter.Param;
import com.github.gkutiel.filter.Filter.SignedValue;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

class Injector {
	private static final Gson GSON = new Gson();

	private static List<Field> fields(final Class<?> c) {
		if (c == null) return new ArrayList<>();
		final List<Field> fields = fields(c.getSuperclass());
		fields.addAll(Arrays.asList(c.getDeclaredFields()));
		return fields;
	}

	private final Filter filter;

	private final HttpServletRequest req;
	private final HttpServletResponse res;

	private final Field[] fields;

	Injector(final Filter filter, final HttpServletRequest req, final HttpServletResponse res) {
		this.filter = filter;
		this.req = req;
		this.res = res;
		final List<Field> fields = fields(filter.getClass());
		this.fields = fields.toArray(new Field[fields.size()]);
		AccessibleObject.setAccessible(this.fields, true);
	}

	private void injectCookie(final Field field) throws JsonSyntaxException, IllegalArgumentException, IllegalAccessException {
		final String val = getCookie(req, field.getName());
		if (val == null) return;
		final SignedValue signedValue = GSON.fromJson(val, SignedValue.class);
		if (!signedValue.isValid()) return;
		field.set(filter, GSON.fromJson(signedValue.val(), field.getType()));
	}

	private void injectVal(final Field field) throws IllegalAccessException {
		final String val = req.getParameter(field.getName());
		if (val == null) return;
		field.set(filter, GSON.fromJson(val, field.getType()));
	}

	Filter inject() throws IllegalArgumentException, IllegalAccessException {
		for (final Field field : fields) {
			if (field.getType().equals(HttpServletRequest.class)) field.set(filter, req);
			if (field.getType().equals(HttpServletResponse.class)) field.set(filter, res);
			if (field.isAnnotationPresent(Param.class)) injectVal(field);
			if (field.isAnnotationPresent(Cookie.class)) injectCookie(field);
		}

		return filter;
	}

}
