package com.github.gkutiel.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.HandlerList;
import org.eclipse.jetty.server.handler.ResourceHandler;
import org.eclipse.jetty.servlet.ServletHandler;
import org.eclipse.jetty.servlet.ServletHolder;

import com.github.gkutiel.filter.Filter.RedirectException;

public class Server {
	public class Map {
		final String path;
		Class<? extends Filter> filter;

		public Map(final String path) {
			this.path = path;
		}

		public Server to(final Class<? extends Filter> filter) {
			this.filter = filter;
			maps.add(this);
			return Server.this;
		}
	}

	private final int port;
	private final List<Map> maps = new ArrayList<>();

	public Server(final int port) {
		this.port = port;
	}

	public Map map(final String path) {
		return new Map(path);
	}

	public void start() {
		try {
			final org.eclipse.jetty.server.Server server = new org.eclipse.jetty.server.Server(port);

			final HandlerList handlers = new HandlerList();
			handlers.addHandler(resourceHandler());
			handlers.addHandler(addServlets());

			server.setHandler(handlers);
			server.start();
			server.join();
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}

	private ServletHandler addServlets() {
		final ServletHandler servletHandler = new ServletHandler();

		for (final Map map : maps) {
			final ServletHolder servletHolder = new ServletHolder(new HttpServlet() {
				@Override
				protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
					try {
						new Engine(req, resp).invoke(map.filter);
					} catch (final RedirectException e) {}
				}

				@Override
				protected void doPost(final HttpServletRequest req, final HttpServletResponse res) throws ServletException, IOException {
					doGet(req, res);
				}
			});
			servletHandler.addServletWithMapping(servletHolder, map.path);
		}
		return servletHandler;
	}

	private ResourceHandler resourceHandler() {
		final ResourceHandler resourceHandler = new ResourceHandler() {
			@Override
			public void handle(final String target, final Request b, final HttpServletRequest r, final HttpServletResponse re) throws IOException,
					ServletException {
				final org.eclipse.jetty.util.resource.Resource f = getResource(r);
				if (f == null || !f.exists()) return;
				if (f.isDirectory() && getWelcome(f) == null) return;
				super.handle(target, b, r, re);
			}
		};
		resourceHandler.setResourceBase("web");
		resourceHandler.setWelcomeFiles(new String[] { "index.html", "index.xml" });
		return resourceHandler;
	}
}
